"""User interface components."""

from .display import Display
from .components import (
    HeaderComponent,
    FooterComponent,
    TabsComponent,
    OverviewPanel,
    GPUPanel,
    ProcessPanel,
    MemoryPanel
)

__all__ = [
    'Display',
    'HeaderComponent',
    'FooterComponent',
    'TabsComponent',
    'OverviewPanel',
    'GPUPanel',
    'ProcessPanel',
    'MemoryPanel'
]